Imports NationalInstruments.ModularInstruments.Interop
Module MainModule
    Sub Main()
        Dim numberOfSamples As Integer = 1000
        Dim magnitudeSquared As Double
        Dim accumulator As Double = 0.0
        Dim data As niComplexNumber()
        Dim wfmInfo As niRFSA_wfmInfo

        Try

            ' Initialize a session to the NI-RFSA device
            Using rfsa As niRFSA = New niRFSA("PXI1Slot2", True, False)

                ' Configure NI-RFSA for a simple IQ acquisition 
                rfsa.ConfigureRefClock("OnboardClock", 10000000.0)
                rfsa.ConfigureReferenceLevel("", 0)
                rfsa.ConfigureAcquisitionType(niRFSAConstants.Iq)
                rfsa.ConfigureIQCarrierFrequency("", 1000000000.0)
                rfsa.ConfigureNumberOfSamples("", True, CLng(numberOfSamples))
                rfsa.ConfigureIQRate("", 1000000.0)

                ' Allocate an array of right size for the driver to fill-in data
                data = New niComplexNumber(numberOfSamples - 1) {}
                rfsa.ReadIQSingleRecordComplexF64("", 10, data, CLng(numberOfSamples), wfmInfo)

                ' Do something useful with the data
                ' We will present average power: 10log(((I^2 + Q ^2) / 2R) * 1000), where R = 50 Ohms
                If (numberOfSamples > 0) Then
                    Dim i As Integer
                    For i = 0 To numberOfSamples - 1
                        magnitudeSquared = ((data(i).Real * data(i).Real) + (data(i).Imaginary * data(i).Imaginary))
                        If (magnitudeSquared = 0) Then
                            magnitudeSquared = 0.00000001
                        End If
                        accumulator = (accumulator + (10 * Math.Log10(((magnitudeSquared / (2 * 50)) * 1000))))
                    Next i
                End If
            End Using ' rfsa session will automatically be closed here

            Console.WriteLine(("Average power = " & (accumulator / numberOfSamples) & " dBm" & Environment.NewLine))
        Catch e As System.Runtime.InteropServices.ExternalException
            Console.WriteLine(("ERROR : " & e.Message.ToString))
        End Try
        Console.ReadKey()
    End Sub

End Module