using System;
using System.Collections.Generic;
using System.Text;
using NationalInstruments.ModularInstruments.Interop;

namespace RFSAGettingStartedSpectrum
{
    class Program
    {
        static void Main(string[] args)
        {
            int numberOfSpectralLines;
            Double[] spectrumData = null;
            Double greatestPeakPower = 0;
            Double greatestPeakFrequency = 0;
            niRFSA_spectrumInfo spectrumInfo;

            try
            {
                // Initialize a session to the NI-RFSA device
                using (niRFSA rfsa = new niRFSA("PXI1Slot2", true, false))
                {
                    // Configure NI-RFSA for a simple IQ acquisition 
                    rfsa.ConfigureRefClock("OnboardClock", 10e6);
                    rfsa.ConfigureReferenceLevel("", 0);
                    rfsa.ConfigureAcquisitionType(niRFSAConstants.Spectrum);
                    rfsa.ConfigureSpectrumFrequencyStartStop("", 990e6, 1010e6);
                    rfsa.ConfigureResolutionBandwidth("", 10e3);
                    
                    // Allocate an array of right size for the driver to fill-in the power spectrum data
                    rfsa.GetNumberOfSpectralLines("", out numberOfSpectralLines);
                    spectrumData = new Double[numberOfSpectralLines];
                    rfsa.ReadPowerSpectrumF64("", 10, spectrumData, numberOfSpectralLines, out spectrumInfo);

                    // Do something useful with the data
                    // We will find the highest peak in a bin, which is not the actual highest
                    // peak and frequency we could find in the acquisition. For an accurate
                    // peak search, we can analyze the data with the Spectral Measurements Toolset.
                    for (int i = 0; i < numberOfSpectralLines; ++i)
                    {
                        if ((i == 0) || (spectrumData[i] > greatestPeakPower))
                        {
                            greatestPeakPower = spectrumData[i];
                            greatestPeakFrequency = spectrumInfo.initialFrequency + spectrumInfo.frequencyIncrement * i;
                        }
                    }
                } // rfsa session will automatically be closed here
                Console.WriteLine("The highest peak in a bin is " + greatestPeakPower + "dBm at " + greatestPeakFrequency / 1e6 + " MHz.");
            }
            catch (System.Runtime.InteropServices.ExternalException e)
            {
                Console.WriteLine("ERROR : " + e.Message.ToString());
            }
            Console.ReadKey();
        }
    }
}
